import os
import numpy as np
import iris
import iris.cube
import iris.analysis
import warnings
import numba
import datetime
from numba import jit, njit
from numba import vectorize
from numba import guvectorize
from numba import cuda
from numba import int32, int64, float32, float64
from numba.cuda.cudadrv import driver


def compute_regsum_2D_cpu(var, basins, e1t, e2t):
    regsum = {}
    for basin, mask in basins.items():
        weighted_var = compute_weighted_var_2D(var, mask, e1t, e2t)
        regsum[basin] = np.sum(weighted_var, axis=(1, 2))
    return regsum


def compute_regsum_3D_cpu(var, basins, e1t, e2t, e3t, save3d):
    regsum = {}
    regsum_3D = {}
    for basin, mask in basins.items():
        weighted_var = compute_weighted_var_3D(var, mask, e1t, e2t, e3t)
        regsum[basin] = np.sum(weighted_var, axis=(2, 3))
        if save3d:
            regsum_3D[basin] = np.sum(weighted_var, axis=(1, 2, 3))
        else:
            regsum_3D = None
    return regsum, regsum_3D


@vectorize(['float32(float32, float32, float32, float32, float32)'],
           target='cpu')
def compute_weighted_var_3D(var, mask, e1t, e2t, e3t):
    weighted_var = var*mask*e1t*e2t*e3t
    return weighted_var


@vectorize(['float32(float32, float32, float32, float32)'], target='cpu')
def compute_weighted_var_2D(var, mask, e1t, e2t):
    weighted_var = var*mask*e1t*e2t
    return weighted_var
